#pragma once

// CNMFavoriteTreeCtrl

//#include "resource.h"
#include "ProfileDlg.h"
#include "ChatDlg.h"
#include "MemoSendDlg.h"

#pragma warning(4:4786)		// identifier too long in debug...
#include <map>


class CDragData;

class CNMFavoriteTreeCtrl : public CTreeCtrl
{

	DECLARE_DYNAMIC(CNMFavoriteTreeCtrl)

protected:
	enum EDropHint {
		DROP_BELOW,
		DROP_ABOVE,
		DROP_CHILD,
		DROP_NODROP
	};

	typedef bool (CNMFavoriteTreeCtrl::*method)(HTREEITEM);
	typedef std::map<bool, method>				shiftmap_t;
	typedef std::map<bool, shiftmap_t>			ctrlmap_t;
	typedef std::map<int, ctrlmap_t>			keymap_t;

	typedef std::map<bool, HCURSOR>				cursctrlmap_t;
	typedef std::map<EDropHint, cursctrlmap_t>	cursormap_t;

	typedef std::map<UINT, method>				cmdmap_t;

	keymap_t	m_Keymap;
	cmdmap_t	m_Commandmap;

	static cursormap_t	m_Cursormap;	// only once per session;

	// dragging
	CDragData*	m_pDragData;

public:
	CNMFavoriteTreeCtrl();
	virtual ~CNMFavoriteTreeCtrl();

protected:
	
	afx_msg void OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult);
//	afx_msg void OnBeginrdrag(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnNMRClick(NMHDR *pNMHDR, LRESULT *pResult);
	DECLARE_MESSAGE_MAP()
public:
	
	afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);
	afx_msg void OnTvnBeginlabeledit(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnTvnEndlabeledit(NMHDR *pNMHDR, LRESULT *pResult);	
	CString m_strCurEditLabelText;

	//void SetGroupList(GROUP_LIST&);
	//GROUP_LIST GetGroupList();
	CString GetNewGroupName();
	int GetGroupId(CString strGroupName);
	
	int m_nSelectedImage;				//2012.01.06
	int m_nImage;

protected:
	//std::vector<CString> m_vecGroupList;
//	GROUP_LIST m_GroupList;
//	CProfileDlg* m_pProfileDlg;
	CMemoSendDlg* m_pMemoSend;
	CChatDlg* m_pChatDlg;	

protected:
	int DelFavoriteUser(void);
	int AddGroup(void);
	int DelGroup(CString strGroup);
	int AddFavoriteUser(int nGroupId, CString strUserId);
	BOOL EditLableOff(void);
	bool IsAncestor(HTREEITEM hItem, HTREEITEM hCheck) const;

protected:
	virtual void OnNewItem(HTREEITEM);		
	virtual bool NewItem(TVINSERTSTRUCT & ins);
	virtual void DragMoveItem(HTREEITEM hDrag, HTREEITEM hDrop, EDropHint hint, bool bCopy);
	virtual void DragStart(void);
	virtual void DragStop(void);
	virtual bool DoInsertRoot(HTREEITEM);
	virtual bool DoDeleteItem(HTREEITEM);
	virtual CDragData * CreateDragData(HTREEITEM hDragItem, bool bRightDrag);
	virtual bool CanDropItem(HTREEITEM hDrag, HTREEITEM hDrop, EDropHint hint);
	virtual HTREEITEM CopyItem(HTREEITEM hOrig, HTREEITEM hParent, HTREEITEM hInsertAfter);
	virtual bool CanDeleteItem(HTREEITEM hItem);
public:
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);

	virtual BOOL PreTranslateMessage(MSG* pMsg);
protected:
	virtual void DragMove(void);
	HTREEITEM GetDropTarget(EDropHint & hint);
	virtual void SetDragCursor(HTREEITEM hDropTarget, EDropHint eHint);
	EDropHint GetDropHint(UINT flags);
	virtual void CreateCursorMap(void);
	virtual bool CanDragItem(TVITEM& item);
	virtual void DragEnd(void);
	virtual bool HandleKeyDown(WPARAM wParam, LPARAM lParam);
};

//---------------------------------------------------------------------------
// helper class CDragData
//---------------------------------------------------------------------------

class CDragData {
	protected:		
		CNMFavoriteTreeCtrl & m_Owner;
		HCURSOR			m_hOrigCursor;	// Original cursor (when not in dragging mode)
		HTREEITEM		m_hItemDrag;	// The item being dragged
		HTREEITEM		m_hLastDrop;	// The point where the last DragMove() occured
		USHORT			m_usNoMove;		// helper to auto-expand a collapsed node while dragging an item over it
		CImageList *	m_pDragImg;		// image of the item being dragged
		bool			m_bRightDrag;	// Are we dragging the item with the right mouse button?
		int				m_nDropHint;	// How to drop the item being dragged?
		

	public:
		CDragData(CNMFavoriteTreeCtrl & tree, HTREEITEM hItem, bool bRightDrag)
			: m_Owner(tree)
			, m_hOrigCursor(::GetCursor())
			, m_hItemDrag(hItem)
			, m_hLastDrop(0)
			, m_usNoMove(0)
			, m_pDragImg(0)
			, m_bRightDrag(bRightDrag)
			, m_nDropHint(0)
		{
			// all real initialization is done in StartDrag() (see below)
		}

		virtual ~CDragData() {
					ASSERT(m_pDragImg == 0);
				}

	public:
		virtual void	StartDrag() {
							CreateDragImage();
							m_Owner.SetCapture();
						}

		virtual void	EndDrag() {
							ReleaseDragImage();
							m_Owner.SelectDropTarget(0);
							m_Owner.EnsureVisible(m_Owner.GetSelectedItem());
							::SetCursor(m_hOrigCursor);
							::ReleaseCapture();
						}

		virtual void	CreateDragImage() {
							ASSERT(m_pDragImg == 0);
							m_pDragImg = m_Owner.CreateDragImage(m_hItemDrag);
							if(m_pDragImg) {
								CPoint pt;
								::GetCursorPos(&pt);
								m_Owner.ScreenToClient(&pt);
								m_pDragImg->DragShowNolock(TRUE);  // lock updates and show drag image
								m_pDragImg->SetDragCursorImage(0,CPoint(0,0));  // define the hot spot for the new cursor image
								m_pDragImg->BeginDrag(0,CPoint(0,0));
								m_pDragImg->DragEnter(&m_Owner,pt);
								m_pDragImg->DragMove(pt);
							}
						}

		virtual void	ReleaseDragImage() {
							if(m_pDragImg) {
								DragLeave();
								m_pDragImg->EndDrag();
								delete m_pDragImg;
							}
							m_pDragImg = 0;
						}

		HTREEITEM		GetDragItem() const { return m_hItemDrag; }
		HTREEITEM		GetDropTarget() const { return m_hLastDrop; }
		void			SetDropTarget(HTREEITEM hItem) { m_hLastDrop = hItem; }
		int				GetDropHint() const { return m_nDropHint; }
		void			SetDropHint(int hint) { m_nDropHint = hint; }
		bool			IsRightDragging() const { return m_bRightDrag; }

		void			DragLeave() {
							if(m_pDragImg)
								m_pDragImg->DragLeave(&m_Owner);
						}

		void			DragEnter(CPoint & pt) {
							if(m_pDragImg)
								m_pDragImg->DragEnter(&m_Owner, pt);
						}

		virtual bool	CheckExpand(HTREEITEM hDrop) {
							if(hDrop == m_hLastDrop) {
								if(++m_usNoMove >= 15) {
									// mouse position unchanged for at least 15 circles.
									// If we're standing on a collapsed node - expand it now
									m_usNoMove = 0;
									return true;
								}
							} else {
								m_usNoMove = 0;	// reset counter if mouse really moved...
								m_hLastDrop = hDrop;
							}
							return false;
						}
};

